@@ -1,6 +1,6 @@
 /* ncftp.h
  *
- * Copyright (c) 1996-2002 Mike Gleason, NCEMRSoft.
+ * Copyright (c) 1996-2004 Mike Gleason, NcFTP Software.
  * All rights reserved.
  *
  */
@@ -13,9 +13,10 @@ extern "C"
 {
 #endif	/* __cplusplus */
 
-#define kLibraryVersion "@(#) LibNcFTP 3.1.3 (March 4, 2002)"
+#define kLibraryVersion "@(#) LibNcFTP 3.1.8 (May 26, 2004)"
 
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
+	/* Includes for Windows */
 #	pragma once
 #	pragma warning(disable : 4127)	// warning C4127: conditional expression is constant
 #	pragma warning(disable : 4100)	// warning C4100: 'lpReserved' : unreferenced formal parameter
@@ -157,6 +158,15 @@ extern "C"
 #		define snprintf _snprintf
 #	endif
 #else
+	/* Includes for UNIX */
+#	if (defined(__linux__)) && (! defined(_LARGEFILE64_SOURCE)) && (! defined(NO_LARGEFILE64_SOURCE))
+		/* Define _LARGEFILE64_SOURCE or NO_LARGEFILE64_SOURCE to
+		 * quiet this warning.  Be sure to define this for ALL
+		 * code that uses LibNcFTP.
+		 */
+#		warning "<ncftp.h> is #defining _LARGEFILE64_SOURCE for you"
+#		define _LARGEFILE64_SOURCE 1
+#	endif
 #	include <unistd.h>
 #	include <sys/types.h>
 #	include <sys/stat.h>
@@ -195,7 +205,7 @@ extern "C"
  * It also specifies the minimum version that is binary-compatibile with
  * this version.  (So this may not necessarily be kLibraryVersion.)
  */
-#define kLibraryMagic "LibNcFTP 3.1.3"
+#define kLibraryMagic "LibNcFTP 3.1.8"
 
 #ifndef longest_int
 #define longest_int long long
@@ -328,6 +338,8 @@ typedef struct FTPConnectionInfo {
 	int stalled;				/* Do not modify this field. */
 	int dataTimedOut;			/* Do not modify this field. */
 	int cancelXfer;				/* You may modify this. */
+	int canceling;				/* Use, but do not modify. */
+	int canceled;				/* Use, but do not modify. */
 
 	char actualHost[64];			/* Do not modify this field. */
 	char ip[32];				/* Do not modify this field. */
@@ -346,20 +358,22 @@ typedef struct FTPConnectionInfo {
 	int hasSIZE;				/* Do not modify this field. */
 	int hasMDTM;				/* Do not modify this field. */
 	int hasREST;				/* Do not modify this field. */
+	int hasNLST_a;				/* Do not modify this field. */
 	int hasNLST_d;				/* Do not modify this field. */
-	int hasUTIME;				/* Do not modify this field. */
 	int hasFEAT;				/* Do not modify this field. */
 	int hasMLSD;				/* Do not modify this field. */
 	int hasMLST;				/* Do not modify this field. */
 	int usedMLS;				/* Do not modify this field. */
 	int hasCLNT;				/* Do not modify this field. */
-	int hasRETRBUFSIZE;			/* Do not modify this field. */
-	int hasRBUFSIZ;				/* Do not modify this field. */
-	int hasRBUFSZ;				/* Do not modify this field. */
-	int hasSTORBUFSIZE;			/* Do not modify this field. */
-	int hasSBUFSIZ;				/* Do not modify this field. */
-	int hasSBUFSZ;				/* Do not modify this field. */
-	int hasBUFSIZE;				/* Do not modify this field. */
+	int hasHELP_SITE;			/* Do not modify this field. */
+	int hasSITE_UTIME;			/* Do not modify this field. */
+	int hasSITE_RETRBUFSIZE;		/* Do not modify this field. */
+	int hasSITE_RBUFSIZ;			/* Do not modify this field. */
+	int hasSITE_RBUFSZ;			/* Do not modify this field. */
+	int hasSITE_STORBUFSIZE;		/* Do not modify this field. */
+	int hasSITE_SBUFSIZ;			/* Do not modify this field. */
+	int hasSITE_SBUFSZ;			/* Do not modify this field. */
+	int hasSITE_BUFSIZE;			/* Do not modify this field. */
 	int mlsFeatures;			/* Do not modify this field. */
 	int STATfileParamWorks;			/* Do not modify this field. */
 	int NLSTfileParamWorks;			/* Do not modify this field. */
@@ -377,9 +391,11 @@ typedef struct FTPConnectionInfo {
 	int ctrlSocketR;			/* You may use but not modify/close. */
 	int ctrlSocketW;			/* You may use but not modify/close. */
 	int dataSocket;				/* You may use but not modify/close. */
+	int dataSocketConnected;		/* Do not use or modify. */
 	int leavePass;				/* You may modify this. */
 	int eofOkay;				/* Do not use or modify. */
 	int require20;				/* You may modify this. */
+	int allowProxyForPORT;			/* You may modify this. */
 	int usingTAR;				/* Use, but do not modify. */
 	int serverType;				/* Do not use or modify. */
 	int ietfCompatLevel;			/* Do not use or modify. */
@@ -391,11 +407,15 @@ typedef struct FTPConnectionInfo {
 	struct timeval connectTime;		/* Do not modify. */
 	struct timeval loginTime;		/* Do not modify. */
 	struct timeval disconnectTime;		/* Do not modify. */
+	struct timeval lastCmdStart;		/* Do not modify. */
+	struct timeval lastCmdFinish;		/* Do not modify. */
 	int numDownloads;			/* Do not modify. */
 	int numUploads;				/* Do not modify. */
 	int numListings;			/* Do not modify. */
 
 	int doNotGetStartingWorkingDirectory;	/* You may modify this. */
+	char textEOLN[4];			/* Set automatically per platform. */
+	const char *manualOverrideFeatures;	/* You may modify this. */
 #if USE_SIO
 	char srlBuf[768];
 	SReadlineInfo ctrlSrl;			/* Do not use or modify. */
@@ -471,7 +491,11 @@ typedef struct MLstItem{
 
 #define kDefaultFTPPort			21
 
-#define kDefaultFTPBufSize		32768
+#if ((defined(AIX) || (defined(_AIX))))
+#	define kDefaultFTPBufSize	4096
+#else
+#	define kDefaultFTPBufSize	32768
+#endif
 
 #ifdef USE_SIO
 /* This version of the library can handle timeouts without
@@ -556,7 +580,9 @@ typedef struct MLstItem{
 #define kFirewallUserAtUserPassAtPass		5
 #define kFirewallFwuAtSiteFwpUserPass		6
 #define kFirewallUserAtSiteFwuPassFwp		7
-#define kFirewallLastType			kFirewallUserAtSiteFwuPassFwp
+#define kFirewallUserAtSitePort			8
+#define kFirewallUserAtSitePort2		9
+#define kFirewallLastType			kFirewallUserAtSitePort2
 
 /* For MLSD, MLST, and STAT. */
 #define kPreferredMlsOpts	(kMlsOptType | kMlsOptSize | kMlsOptModify | kMlsOptUNIXmode | kMlsOptUNIXowner | kMlsOptUNIXgroup | kMlsOptUNIXuid | kMlsOptUNIXgid | kMlsOptPerm)
@@ -589,6 +615,7 @@ typedef struct MLstItem{
 #define kConfirmResumeProcSaidCancel 6
 
 typedef int (*FTPConfirmResumeDownloadProc)(
+	const FTPCIPtr cip,
 	const char **localpath,
 	longest_int localsize,
 	time_t localmtime,
@@ -599,6 +626,7 @@ typedef int (*FTPConfirmResumeDownloadProc)(
 );
 
 typedef int (*FTPConfirmResumeUploadProc)(
+	const FTPCIPtr cip,
 	const char *localpath,
 	longest_int localsize,
 	time_t localmtime,
@@ -635,12 +663,15 @@ typedef int (*FTPConfirmResumeUploadProc)(
 #define kServerTypeRoxen		10
 #define kServerTypeNetWareFTP		11
 #define kServerTypeWS_FTP		12
+#define kServerTypeDguxFTP		13
+#define kServerTypeIBMFTPCS		14
+#define kServerTypePyramid		15
 
 #if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
+	/* Windows has separate functions to close and ioctl sockets. */
+#else
+	/* UNIX uses close() and ioctl() for all types of descriptors */
 #	define closesocket close
-#endif
-
-#if (defined(WIN32) || defined(_WINDOWS)) && !defined(__CYGWIN__)
 #	define ioctlsocket ioctl
 #endif
 
@@ -791,12 +822,12 @@ extern int gLibNcFTP_Uses_Me_To_Quiet_Variable_Unused_Warnings;
 #	ifndef UNUSED
 #		define UNUSED(a) a
 #	endif
-#elif (defined(__GNUC__)) && (__GNUC__ >= 3)
+#elif (defined(__GNUC__)) && (__GNUC__ >= 3) && (! defined(__cplusplus))
 #	ifndef UNUSED
 #		define UNUSED(a) a __attribute__ ((__unused__))
 #	endif
 #	define LIBNCFTP_USE_VAR(a)
-#elif (defined(__GNUC__)) && (__GNUC__ == 2)
+#elif (defined(__GNUC__)) && (__GNUC__ == 2) && (! defined(__cplusplus))
 #	ifndef UNUSED
 #		define UNUSED(a) a __attribute__ ((unused))
 #	endif
@@ -821,6 +852,7 @@ int FTPCmd(const FTPCIPtr cip, const char *const cmdspec, ...)
 __attribute__ ((format (printf, 2, 3)))
 #endif
 ;
+int FTPDecodeHostName(const FTPCIPtr cip, const char *const hstr0);
 int FTPDecodeURL(const FTPCIPtr cip, char *const url, FTPLineListPtr cdlist, char *const fn, const size_t fnsize, int *const xtype, int *const wantnlst);
 int FTPDelete(const FTPCIPtr cip, const char *const pattern, const int recurse, const int doGlob);
 int FTPFileExists(const FTPCIPtr cip, const char *const file);
@@ -847,6 +879,7 @@ int FTPOpenHostNoLogin(const FTPCIPtr cip);
 void FTPPerror(const FTPCIPtr cip, const int err, const int eerr, const char *const s1, const char *const s2);
 int FTPPutFiles3(const FTPCIPtr cip, const char *const pattern, const char *const dstdir1, const int recurse, const int doGlob, const int xtype, int appendflag, const char *const tmppfx, const char *const tmpsfx, const int resumeflag, const int deleteflag, const FTPConfirmResumeUploadProc resumeProc, int UNUSED(reserved));
 int FTPPutOneFile3(const FTPCIPtr cip, const char *const file, const char *const dstfile, const int xtype, const int fdtouse, const int appendflag, const char *const tmppfx, const char *const tmpsfx, const int resumeflag, const int deleteflag, const FTPConfirmResumeUploadProc resumeProc, int UNUSED(reserved));
+int FTPReadLoginConfigFile(FTPCIPtr cip, const char *const fn);
 int FTPRemoteGlob(FTPCIPtr cip, FTPLineListPtr fileList, const char *pattern, int doGlob);
 int FTPRename(const FTPCIPtr cip, const char *const oldname, const char *const newname);
 int FTPRmdir(const FTPCIPtr cip, const char *const pattern, const int recurse, const int doGlob);
@@ -887,14 +920,17 @@ char *FGets(char *, size_t, FILE *);
 void GetHomeDir(char *, size_t);
 void GetUsrName(char *, size_t);
 void Scramble(unsigned char *dst, size_t dsize, unsigned char *src, char *key);
+double Duration(struct timeval *t0);
 struct tm *Gmtime(time_t t, struct tm *const tp);
 struct tm *Localtime(time_t t, struct tm *const tp);
 time_t GetUTCOffset(const int mon, const int mday);
+time_t GetUTCOffset2(const int year, const int mon, const int mday, const int hour, const int min);
 time_t UnMDTMDate(char *);
 int MkDirs(const char *const, int mode1);
 char *GetPass(const char *const prompt, char *const pwbuf, const size_t pwbufsize);
 int FilenameExtensionIndicatesASCII(const char *const pathName, const char *const extnList);
 void StrRemoveTrailingSlashes(char *dst);
+int StrToBoolOrInt(const char *s);
 #if defined(WIN32) || defined(_WINDOWS) || defined(__CYGWIN__)
 char *StrFindLocalPathDelim(const char *src);
 char *StrRFindLocalPathDelim(const char *src);
@@ -930,8 +966,18 @@ int FTPPutFileFromMemory(const FTPCIPtr cip, const char *volatile dstfile, const
 
 /* Everything else below are private routines, or stuff for testing */
 int FTPInitConnectionInfo2(const FTPLIPtr lip, const FTPCIPtr cip, char *const buf, size_t bufSize);
+int FTPSendPort(const FTPCIPtr cip, struct sockaddr_in *saddr);
+int FTPSendPassive(const FTPCIPtr cip, struct sockaddr_in *saddr, int *weird);
+int FTPSetStartOffset(const FTPCIPtr cip, longest_int restartPt);
+void FTPCloseControlConnection(const FTPCIPtr cip);
+int FTPSendCommand(const FTPCIPtr cip, const char *const cmdspec, va_list ap)
+#if (defined(__GNUC__)) && (__GNUC__ >= 2)
+__attribute__ ((format (printf, 2, 0)))
+#endif
+;
 char *FTPGetLocalCWD(char *buf, size_t size);
 int FTPQueryFeatures(const FTPCIPtr);
+void FTPManualOverrideFeatures(const FTPCIPtr cip);
 int FTPMListOneFile(const FTPCIPtr cip, const char *const file, const MLstItemPtr mlip);
 void FTPInitializeAnonPassword(const FTPLIPtr);
 void FTPInitIOTimer(const FTPCIPtr);
@@ -973,6 +1019,7 @@ int FTPFileExistsStat(const FTPCIPtr cip, const char *const file);
 int FTPFileExistsNlst(const FTPCIPtr cip, const char *const file);
 int FTPFileExists2(const FTPCIPtr cip, const char *const file, const int tryMDTM, const int trySIZE, const int tryMLST, const int trySTAT, const int tryNLST);
 
+void FTPGetDateStr(time_t t, const char *fmt, char *const ltstr1, const size_t ltstr1size, char *const gtstr1, const size_t gtstr1size);
 int BufferGets(char *, size_t, int, char *, char **, char **, size_t);
 void DisposeFileInfoListContents(FTPFileInfoListPtr);
 void InitFileInfoList(FTPFileInfoListPtr);
@@ -993,9 +1040,9 @@ int ConcatFileToFileInfoList(FTPFileInfoListPtr, char *);
 int LineListToFileInfoList(FTPLineListPtr, FTPFileInfoListPtr);
 int LineToFileInfoList(FTPLinePtr, FTPFileInfoListPtr);
 void URLCopyToken(char *, size_t, const char *, size_t);
-int UnMlsT(const char *const, const MLstItemPtr);
-int UnMlsD(FTPFileInfoListPtr, FTPLineListPtr);
-int UnLslR(FTPFileInfoListPtr, FTPLineListPtr, int);
+int UnMlsT(const FTPCIPtr, const char *const, const MLstItemPtr);
+int UnMlsD(const FTPCIPtr, FTPFileInfoListPtr, FTPLineListPtr);
+int UnLslR(const FTPCIPtr, FTPFileInfoListPtr, FTPLineListPtr, int);
 void TraceResponse(const FTPCIPtr, ResponsePtr);
 void PrintResponse(const FTPCIPtr, FTPLineListPtr);
 void DoneWithResponse(const FTPCIPtr, ResponsePtr);
@@ -1008,6 +1055,16 @@ int RCmd(const FTPCIPtr, ResponsePtr, const char *, ...)
 __attribute__ ((format (printf, 3, 4)))
 #endif
 ;
+void PrintF(const FTPCIPtr cip, const char *const fmt, ...)
+#if (defined(__GNUC__)) && (__GNUC__ >= 2)
+__attribute__ ((format (printf, 2, 3)))
+#endif
+;
+void FTPLogError(const FTPCIPtr cip, const int pError, const char *const fmt, ...)
+#if (defined(__GNUC__)) && (__GNUC__ >= 2)
+__attribute__ ((format (printf, 3, 4)))
+#endif
+;
 
 #ifdef __cplusplus
 }
